﻿
-- The minimum cooldown duration to make the button fading out
local MIN_DURATION = 1.5

-- Alpha value for completly faded out button
local FADED_ALPHA_VALUE = 0.15

-- Globals made upvalues
local _G = _G
local GetTime = GetTime
local next = next
local pairs = pairs

-- Locals
local updateFrame
local fadedFrames = {}

local function endCooldown(frame) 
	if not updateFrame or not fadedFrames[frame] or not frame.CF_alpha then
		return
	end	
	frame:SetAlpha(frame.CF_alpha)
	frame.CF_alpha = nil
	frame.CF_end = nil
	frame.CF_duration = nil
	fadedFrames[frame] = nil
	if not next(fadedFrames) then
		updateFrame:Hide()
	end
end

local function updateAlpha(frame)
	local remaining = frame.CF_end - GetTime()
	if remaining <= 0 then
		endCooldown(frame)			
	else 
		local alpha = FADED_ALPHA_VALUE + (1.0 - FADED_ALPHA_VALUE) * (1.0 - remaining / frame.CF_duration) ^ 3
		frame:SetAlpha(frame.CF_alpha * alpha)
	end
end

local function onUpdate(updateFrame, elapsed)
	updateFrame.timeToUpdate = updateFrame.timeToUpdate - elapsed
	if updateFrame.timeToUpdate > 0 then
		return
	else
		updateFrame.timeToUpdate = 0.1
	end
	if not next(fadedFrames) then
		updateFrame:Hide()
		return
	end
	for frame in pairs(fadedFrames) do
		updateAlpha(frame)
	end
	
end

local function CreateUpdateFrame() 	
	local frame = CreateFrame("Frame", nil, UIParent)
	frame:SetScript('OnUpdate', onUpdate)
	return frame
end

local function updateCooldown(cooldownFrame, start, duration, enable)
	local frame = cooldownFrame:GetParent()
	
	if frame:IsVisible() and start > 0 and duration > MIN_DURATION and enable > 0 then
		if not updateFrame then
			updateFrame = CreateUpdateFrame()
		end
		if not fadedFrames[frame]  then
			fadedFrames[frame] = true
			frame.CF_alpha = frame:GetAlpha()
		end
		frame.CF_duration = duration
		frame.CF_end = start + duration
		updateFrame.timeToUpdate = 0
		updateFrame:Show()
	else
		endCooldown(frame)
	end
	
end

hooksecurefunc("CooldownFrame_SetTimer", updateCooldown)
